# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2016 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views for employee schedules
"""

from __future__ import unicode_literals, absolute_import

import datetime

from rattail.db import model
from rattail.time import localtime, make_utc

from pyramid_simpleform import Form

from tailbone.db import Session
from tailbone.views.shifts.lib import TimeSheetView, ShiftFilter


class ScheduleView(TimeSheetView):
    """
    Simple view for current user's schedule.
    """
    key = 'schedule'
    model_class = model.ScheduledShift

    def edit(self):
        """
        View for editing (full) schedule.
        """
        # first process filters; will redirect if any were received
        form = Form(self.request, schema=ShiftFilter)
        self.process_filter_form(form)

        # okay then, maybe process saved shift data
        if self.request.method == 'POST':

            # organize form data by uuid / field
            fields = ['employee_uuid', 'store_uuid', 'start_time', 'end_time', 'delete']
            data = dict([(f, {}) for f in fields])
            for key in self.request.POST:
                for field in fields:
                    if key.startswith('{}-'.format(field)):
                        uuid = key[len('{}-'.format(field)):]
                        if uuid:
                            data[field][uuid] = self.request.POST[key]
                        break

            # apply delete operations
            deleted = []
            for uuid, value in data['delete'].iteritems():
                assert value == 'delete'
                shift = Session.query(model.ScheduledShift).get(uuid)
                assert shift
                Session.delete(shift)
                deleted.append(uuid)

            # apply create / update operations
            created = {}
            updated = {}
            time_format = '%a %d %b %Y %I:%M %p'
            for uuid, employee_uuid in data['start_time'].iteritems():
                if uuid in deleted:
                    continue
                if uuid.startswith('new-'):
                    shift = model.ScheduledShift()
                    shift.employee_uuid = data['employee_uuid'][uuid]
                    shift.store_uuid = data['store_uuid'][uuid]
                    Session.add(shift)
                    created[uuid] = shift
                else:
                    shift = Session.query(model.ScheduledShift).get(uuid)
                    assert shift
                    updated[uuid] = shift
                start_time = datetime.datetime.strptime(data['start_time'][uuid], time_format)
                shift.start_time = make_utc(localtime(self.rattail_config, start_time))
                end_time = datetime.datetime.strptime(data['end_time'][uuid], time_format)
                shift.end_time = make_utc(localtime(self.rattail_config, end_time))

            self.request.session.flash("Changes were applied: created {}, updated {}, "
                                       "deleted {} Scheduled Shifts".format(
                                           len(created), len(updated), len(deleted)))
            return self.redirect(self.request.route_url('schedule.edit'))

        context = self.get_timesheet_context()
        context['form'] = form
        context['page_title'] = "Edit Schedule"
        return self.render_full(**context)

    @classmethod
    def defaults(cls, config):
        cls._defaults(config)

        # edit schedule
        config.add_route('schedule.edit', '/schedule/edit')
        config.add_view(cls, attr='edit', route_name='schedule.edit',
                        renderer='/shifts/schedule_edit.mako',
                        permission='schedule.edit')
        config.add_tailbone_permission('schedule', 'schedule.edit', "Edit full schedule")


def includeme(config):
    ScheduleView.defaults(config)
